from typing import List, Optional

from sqlalchemy.orm import Session

from app.models.todo import Todo
from app.schemas.todo import TodoCreate, TodoUpdate


def get_todo(db: Session, todo_id: int) -> Optional[Todo]:
    return db.query(Todo).filter(Todo.id == todo_id).first()


def get_todos(
    db: Session, 
    owner_id: int, 
    skip: int = 0, 
    limit: int = 100,
    title: Optional[str] = None,
    is_completed: Optional[bool] = None
) -> List[Todo]:
    query = db.query(Todo).filter(Todo.owner_id == owner_id)
    
    if title:
        query = query.filter(Todo.title.contains(title))
    
    if is_completed is not None:
        query = query.filter(Todo.is_completed == is_completed)
        
    return query.offset(skip).limit(limit).all()


def create_todo(db: Session, todo_in: TodoCreate, owner_id: int) -> Todo:
    db_todo = Todo(
        title=todo_in.title,
        description=todo_in.description,
        is_completed=todo_in.is_completed,
        owner_id=owner_id
    )
    db.add(db_todo)
    db.commit()
    db.refresh(db_todo)
    return db_todo


def update_todo(
    db: Session, db_obj: Todo, obj_in: TodoUpdate
) -> Todo:
    update_data = obj_in.model_dump(exclude_unset=True)
    for field in update_data:
        setattr(db_obj, field, update_data[field])
    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def delete_todo(db: Session, todo_id: int) -> Todo:
    todo = db.query(Todo).filter(Todo.id == todo_id).first()
    db.delete(todo)
    db.commit()
    return todo